@@ -13,24 +13,40 @@ is available at U{http://www.opensource.org/licenses/bsd-license.php}
 import gtk
 from accerciser.plugin import ViewportPlugin
 from accerciser.i18n import _, N_
-from pyLinAcc import *
+import pyatspi
 
-class DemoViewport(ViewportPlugin):
+class APIBrowser(ViewportPlugin):
+  '''
+  Plugin class for API Browser.
+
+  @ivar iface_combo: Combobox that shows available interfaces.
+  @type iface_combo: gtk.ComboBox
+  @ivar method_tree: Tree view with available methods from chosen interface.
+  @type method_tree: gtk.TreeView
+  @ivar property_tree: Tree view with available properties from chosen 
+  interface.
+  @type property_tree: gtk.TreeView
+  @ivar private_toggle: Toggles visibility of private attributes.
+  @type private_toggle: gtk.CheckButton
+  '''
   plugin_name = N_('API Browser')
+  plugin_name_localized = _(plugin_name)
   plugin_description = \
       N_('Browse the various methods of the current accessible')
   def init(self):
+    '''
+    Initialize the API browser plugin.
+    '''
     self._buildUI()
-    self._initModels()
+    self._initTreeViews()
     self.iface_combo.connect('changed', self._refreshAttribs)
     self.private_toggle.connect('toggled', self._refreshAttribs)
     self.curr_iface = None
-    self.constructors = {}
-    for iface in dir(Interfaces):
-      if iface.startswith('I'):
-        self.constructors[iface[1:].lower()] = getattr(Interfaces, iface)
 
   def _buildUI(self):
+    '''
+    Manually build the plugin's UI.
+    '''
     vbox = gtk.VBox()
     self.plugin_area.add(vbox)
     top_hbox = gtk.HBox()
@@ -52,7 +68,10 @@ class DemoViewport(ViewportPlugin):
     bottom_hbox.pack_end(self.private_toggle, False)
     self.show_all()
     
-  def _initModels(self):
+  def _initTreeViews(self):
+    '''
+    Initialize the properties and methods tree views and models.
+    '''
     # method view
     model = gtk.ListStore(str, str)
     self.method_tree.set_model(model)
@@ -77,37 +96,41 @@ class DemoViewport(ViewportPlugin):
     self.property_tree.append_column(tvc)
 
   def onAccChanged(self, acc):
+    '''
+    Update the UI when the selected accessible changes.
+    
+    @param acc: The applications-wide selected accessible.
+    @type acc: Accessibility.Accessible
+    '''
     self.acc = acc
-    ints = self._getInterfaces(acc)
+    ints = pyatspi.listInterfaces(acc)
     model = self.iface_combo.get_model()
     model.clear()
     for iface in ints:
       self.iface_combo.append_text(iface)
     self.iface_combo.set_active(0)
-
-  def _getInterfaces(self, acc):
-    ints = []
-    for func in (f for f in Interfaces.__dict__.values() 
-                 if callable(f) and f.func_name.startswith('I')):
-      try:
-        i = func(acc)
-      except Exception:
-        pass
-      else:
-        ints.append(func.func_name[1:].lower())
-    ints.sort()
-    return ints
   
   def _refreshAttribs(self, widget):
+    '''
+    Refresh the attributes in the tree views. Could be used as a callback.
+    
+    @param widget: The widget that may have triggered this callback.
+    @type widget: gtk.Widget
+    '''
     iface = self.iface_combo.get_active_text()
     try:
-      self.curr_iface = self.constructors[iface](self.acc)
-    except KeyError:
+      query_func = getattr(self.acc, 'query'+iface)
+    except AttributeError:
       pass
     else:
+      self.curr_iface = query_func()
       self._popAttribViews()
 
   def _popAttribViews(self):
+    '''
+    Populate the attribute views with information from currently selected 
+    accessible and interface.
+    '''
     prop_model = self.property_tree.get_model()
     method_model = self.method_tree.get_model()
     prop_model.clear()
